% Kompensation des Phasen- und Amplitudenfehlers durch die parasitre
% Shuntinduktivitt & digitaler Tiefpassfilter im Frequenzbereich
% nach Marc Hauck, Masterarbeit, 2015
% Kompensation angepasst: 25.01.2018, Nikolaus Trnka
%
% Inputs:   Fa - Abtastfrequenz in Hz
%           y  - Stromverlauf, bereits in A umgerechnet
%           R  - Widerstand des Shunts, bei R<0 keine Kompensation, nur Filterung
%           L  - parasitre Induktivitt des Shunts in H (heha 170110)
%     cutfreq  - cutoff freq. in Hz, Filter deaktivieren mit sehr hoher Frequenz
%
% Outputs:  yc - korrigierter und gefilterter Stromverlauf in A
%
% Stand: 25.01.2018

function yc = freq_correction_neu2(Fa, y, R, cutfreq)
    
    
    
    % berechne fft des Stromverlaufs, schneide DC-Anteil ab
    [FFTY,fplot]=FFT(y,Fa,'AC','fig'); %,'fig'
    
    % jetzt fftshift, um Spektrum zu verschieben. Frequenz geht nicht mit
    % fftshift, daher fplot von Hand ndern ( Verschieben um fmax/2)
    % Versuch: da ber 'symmetic' bei IFFT nur erste Hlfte bentigt wird,
    % erstmal nur eine Hlfte versuchen, zweite Hlfte stimmt nicht, wird
    % aber auch nicht in IFFT verwendet
    %freq=[fplot(1); fplot(2:ceil((length(fplot)-1)/2)+1); flipud(fplot(2:floor((length(fplot)-1)/2)+1))]; 

    % schneide hohe Frequenzen ab
    FFTY(fplot>cutfreq)=0;
    
    % Indizes fr der Fitdaten fr Shunts ber log10(1/R) aus R berechnen
    
	% Signal korrigieren
	if R > 0 
        % Lade Fitdaten
        load('fitdataFreq','-mat','f');
        load(['fitdataR' num2str(R*1000)],'-mat','GSh');
        % Frequenzgang des Sunts ber slmFit-Toolbox
        %GSh = slmeval(fplot,slmFit{log10(1/R)}{1}).*exp(1j.*slmeval(fplot,slmFit{log10(1/R)}{2}));
        % GSh an richtigen Frequenzen auswerten
        GShfreq=interp1(f,GSh,fplot,'pchip');
        % Dekonvolution: Division der Frequenzgnge von Signal und Shunt
		FFTY = FFTY ./ GShfreq;          
	end

	% Rcktransformation des korrigierten Signals mit defaultoption
	% 'symmetric' - nur so kann zweite Hlfte des Spektr. ignoriert werden
	% Angabe von fplot nicht ntig, da in LabView altes t genutzt wird
	yc = IFFT(FFTY);   
    
%     % Testplot
%     t=linspace(0,length(y)-1,length(y))./Fa;
%     figure; plot(t,y,t,yc)
end
