%% Hysteresis model after Takacs, Meszaros
% Get field strength from hysteresis model

% Mathias Lindner
% March 2014


%%

function h_new_dir = func_h_from_j(j_new, par_c, n_par_h)
	%% Analyze polarization
	% Derivative
	t_new = 1:length(j_new);
	djdt_new = gradient(j_new)./gradient(t_new);
	% split into rising and falling
	j_new_up = djdt_new > 0;
	% preallocate output arrays
	h_new_dir = zeros(size(j_new));
	z_dir = zeros(size(j_new));
	
	%% Get field strength range
	% Build data basis to interpolate from
	h_normal = -20e3:10:20e3;
	h_normal = [-200e3:5e3:-25e3 h_normal 25e3:5e3:200e3];
	% get normal magnetization curve as maximum loci 
	j_normal = 0.5 * (func_hysteresis_up(par_c, h_normal, n_par_h) + ...
		func_hysteresis_dn(par_c, h_normal, n_par_h));
	%remove the area where polarization remains constant
	h_normal(diff(j_normal)<1e-5) = [];
	j_normal(diff(j_normal)<1e-5) = [];
	% find maximum field strength for requested polarization
	h_max = interp1(j_normal, h_normal, max(j_new), 'linear', 'extrap');
	h_min = interp1(j_normal, h_normal, min(j_new), 'linear', 'extrap');
	% limit field strength (in case the material does not allow
	% such high polarizations)
	if h_max > 200e3, h_max=200e3; end
	if h_min < -200e3, h_min=-200e3; end
	
	%% Get new waveform
	% Calculate hysteresis curves
	h_hyst_up = h_min:h_max/1e5:h_max;
	h_hyst_dn = h_hyst_up;
	j_hyst_up = func_hysteresis_up(par_c, h_hyst_up, n_par_h);
	j_hyst_dn = func_hysteresis_dn(par_c, h_hyst_dn, n_par_h);
	% Interpolate new field strength from hysteresis
	h_new_dir(j_new_up) = ...
		interp1(j_hyst_up, h_hyst_up, j_new(j_new_up), 'pchip', 'extrap');
	h_new_dir(~j_new_up) = ...
		interp1(j_hyst_dn, h_hyst_dn, j_new(~j_new_up), 'pchip', 'extrap');
end

