%% Hysteresis model after Takacs
% Fit function for static function

% Mathias Lindner
% September 2015

	%%
	function [J_fit, Jacobian] = func_static(par_s, H, varargin)
		% Parameters according Meszaros2008
		
		% set Offset to zero if not given (needed for heredity part)
		if nargin==2
			H_off = zeros(length(par_s)/2, 1);
		else
			% unused offset zero because reversible part
			H_off = [varargin{1}; zeros(length(par_s)/2-length(varargin{1}), 1)];
		end

		% apply static funtion
		% at least three components: ferrite, martensite, reversible part
		J_fit = 0;
		for i=1:length(par_s)/2
			J_fit = J_fit + par_s(2*i-1)*tanh(par_s(2*i)*(H+H_off(i)));
		end
		% add vacuum magnetization to extrapolate for high field strengths
		% added 2015: vacuum not sufficient for extrapolation, added another term with
		% approx. mu_r = 20
		J_fit = J_fit + 4*pi*1e-7*H + 3e-5*H;
		
		% calc Jacobian matrix if neccessary
		if nargout == 2
			Jacobian = zeros(length(H),length(par_s));
			for i=1:length(par_s)/2
				Jacobian(:,2*i-1) = tanh(par_s(2*i)*(H'+H_off(i)));
				Jacobian(:,2*i) = par_s(2*i-1)*(H'+H_off(i)) ./ ((cosh(par_s(2*i)*(H'+H_off(i)))).^2);
			end
		end
	end
		
		
		
