%% Hysteresis model after Takacs
% Postprocessing

% Mathias Lindner
% March 2014


%% average branches via field strength for better interpolation
% make polarizations unique for interpolation
[b_dn_unique, idx_unique] = unique(b_dn);
h_dn_unique = h_dn(idx_unique);
% interpolate and average
h_static = mean([h_up; interp1(b_dn_unique,h_dn_unique,b_up,'pchip')],1);

% invert negative values since function not suitable for larger
% negative field strengths
h_neg = find(h_static<0);
h_static_abs = abs(h_static);
b_static_abs = b_up;
b_static_abs(h_neg) = -1 * b_up(h_neg);


	%% Plot measurement data vs. fit
	% Parameter for only plotting parts of the static curve
	%par_tanh = par_s; par_tanh(1) = 0;
	%par_e = par_s; par_e(2) = 0;
	%par_korr = par; par_korr(1) = 0; par_korr(2) = 0;
	%h_pos = find(h>=0);
	% Plot
%	for i = 1:size(b,1)
	i = 1;
%		subplot(2*size(b,1),1,2*i-1)
	figure(7)
		plot(h(i,field_up(i,:)), b(i,field_up(i,:)), 'b.', ...
			h(i,~field_up(i,:)), b(i,~field_up(i,:)), 'c.', ...
			h(i,field_up(i,:)), ...
				func_hysteresis_up(par_h_c,h(i,field_up(i,:)),n_par_h), 'k.', ...
			h(i,~field_up(i,:)), ...
				func_hysteresis_dn(par_h_c,h(i,~field_up(i,:)),n_par_h), 'r.', ...
			h(i,:), func_static(par_s, h(i,:)), 'g.')
		grid
		legend('measured up', 'measured dn', 'fit up', 'fit dn', 'anhysteric', 'Location', 'SouthEast')
		title('Fitting behaviour of hysteresis')
%		subplot(2*size(b,1),1,2*i)
% 	figure(3)
% 		plot(b(i,field_up(i,:)), h(i,field_up(i,:)) - ...
% 				func_static_inverse(par_s,b(i,field_up(i,:))), 'r.', ...
% 			b(i,~field_up(i,:)), h(i,~field_up(i,:)) - ...
% 				func_static_inverse(par_s,b(i,~field_up(i,:))), 'y.', ...
% 			b(i,field_up(i,:)), ...
% 				fit_hysteresis_ext_ampkorr_fup(par_h, b(field_up), b_max,b_min, par_s) - ...
% 				func_static_inverse(par_s,b(i,field_up(i,:))), 'm.', ...
% 			b(i,~field_up(i,:)), ...
% 				fit_hysteresis_ext_ampkorr_fdown(par_h, b(~field_up), b_max,b_min, par_s) - ...
% 				func_static_inverse(par_s,b(i,~field_up(i,:))), 'm.')
%			func_static(par_tanh, h(i,:)), h(i,:), 'g.', ...
%			func_static(par_e, h(i,:)), h(i,:), 'm.', ...
%		
% 		grid
	figure(5)
		plot(func_static(par_s, abs(h)), abs(h), 'k', ...
			par_s(1)*tanh(par_s(2)*abs(h)),abs(h),'b', ...
			par_s(3)*tanh(par_s(4)*abs(h)),abs(h),'r', ...
			par_s(5)*tanh(par_s(6)*abs(h)),abs(h),'m')%, ...
			%par_s(7)*tanh(par_s(8)*abs(h)),abs(h),'g')
		grid
		legend('anhysteric', 'part 1', 'part 2', 'part 3', 'part 4', 'Location', 'SouthEast')
		title('Components anhysteric function')
	figure(6)
		plot(b(i,field_up(i,:)), h(i,field_up(i,:)), 'b.', ...
			b(i,~field_up(i,:)), h(i,~field_up(i,:)), 'c.', ...
			b_static_abs, h_static_abs, 'k.', ...
			func_static(par_s, h(i,:)), h(i,:), 'g.', ...
			func_static(par_s_ms, h(i,:)), h(i,:), 'r.')
		grid
		legend('measured up', 'measured dn', 'mean curve', 'anhysteric', 'global fit mean curve', 'Location', 'SouthEast')
		title('Anhysteric function')

%	end
	
	
	%% Define new expected induction loop
	% Time and induction
	t_new = 0:1e-4:40e-3;
	b_new = 0.5 * sin(2*pi*50*t_new);
% 	
	% Get field strength from hysteresis model
	h_new_dir = func_h_from_j(b_new, par_h_c, n_par_h);

	
	% Plot
	figure(2)
	plotyy(t_new, h_new_dir, t_new, b_new)
	grid
	
	figure(4)
	plot(h_new_dir, b_new, func_static_inverse(par_s, b_new), b_new, ...
		h(i,field_up(i,:)), b(i,field_up(i,:)), 'k.', ...
			h(i,~field_up(i,:)), b(i,~field_up(i,:)), 'k.', ...
			'LineWidth', 1, 'MarkerSize', 4)
	grid
	legend('new hysteresis', 'anhysteric', 'old measured hysteresis', 'Location', 'SouthEast')
	
	
	
	%% Data export
% 	data_up = [b(field_up)', h(field_up)', ...
% 		fit_hysteresis_ext_fup(par_h,b,field_up,b_max,b_min,1,par_s)', ...
% 		func_static(par_s, h(field_up))', ...
% 		(fit_hysteresis_ext_fup(par_h, b, field_up, b_max,b_min, 1, par_s) - ...
% 			func_static_inverse(par_s,b(field_up)))'];
% 	data_down = [b(~field_up)', h(~field_up)', ...
% 		fit_hysteresis_ext_fdown(par_h,b,field_up,b_max,b_min,1,par_s)', ...
% 		func_static(par_s, h(~field_up))', ...
% 		(fit_hysteresis_ext_fdown(par_h, b, field_up, b_max,b_min, 1, par_s) - ...
% 			func_static_inverse(par_s,b(~field_up)))'];
% 	% Daten mit J sortieren
% 	[data_up_sort, idx] = sort(data_up);
% 	data_up_sort = data_up(idx(:,1),:);
% 	[data_down_sort, idx] = sort(data_down);
% 	data_down_sort = data_down(idx(:,1),:);
% 	% Dateien schreiben
% 	dlmwrite('hysteresis_up.csv',data_up_sort,'\t');
% 	dlmwrite('hysteresis_down.csv',data_down_sort,'\t');