%% Hysteresis model after Takacs, Meszaros
% static curve as from Magnettechnik Kleine (tanh function)
% Preprocessing

% Mathias Lindner
% March 2014


%% Data conditioning
h_up = h(field_up);
b_up = b(field_up);
h_dn = h(~field_up);
b_dn = b(~field_up);

% make polarizations unique for interpolation
[b_dn_unique, idx_unique] = unique(b_dn);
h_dn_unique = h_dn(idx_unique);
% interpolate and average
h_static = mean([h_up; interp1(b_dn_unique,h_dn_unique,b_up,'pchip')],1);

% invert negative values since function not suitable for larger
% negative field strengths
h_neg = find(h_static<0);
h_static_abs = abs(h_static);
b_static_abs = b_up;
b_static_abs(h_neg) = -1 * b_up(h_neg);

% error weighting
weight_s_max = 2;
weight_s(b_up>=0) = weight_s_max * b_up(b_up>=0)/max(b_up);
weight_s(b_up<0) =  weight_s_max * b_up(b_up<0)/min(b_up);
weight_s(weight_s<1) = 1;

% delete h=0 points since jacobian matrix is not defined there
b_static_abs(h_static_abs==0) = [];
weight_s(h_static_abs==0) = [];
h_static_abs(h_static_abs==0) = [];




%% Estimated optimization parameters
% anhysteric curve
% amount of material phases
n_par_s = 3;
% static curve
par_start_s = reshape(	[1.0; 1e-3]		* ones(1,n_par_s), 2*n_par_s,1);
par_lb_s =	reshape(	[1e-9; 1e-9]	* ones(1,n_par_s), 2*n_par_s,1);
par_ub_s =	reshape(	[3; 0.1]			* ones(1,n_par_s), 2*n_par_s,1);
		% ub 0.1 for minimum 20-30 A/m until tanh reaches saturation
		% otherwise jumps in overall curve appearance

% if no global optimization to be done, set starting parameters for local optimizer
par_s_ms = par_start_s;


% hysteresis curve
% coercivity as initial guess
h_c = mean(abs(h(abs(b) < 0.1)));
% amount of parameters
n_par_h = 3;
par_start_h =	h_c * ones(n_par_h,1);
par_lb_h =		1e-9 * ones(n_par_h,1);
par_ub_h =		1e6 * ones(n_par_h,1);

% hysteresis only fitted together with anhysteric curve
par_h_ms = [par_start_h; par_start_s];
par_lb_h = [par_lb_h; par_lb_s];
par_ub_h = [par_ub_h; par_ub_s];



%% Global fit options
% anhysteric
	% options for each local optimization in MultiStart
	ms_options = optimset('MaxIter', 70, 'Jacobian','on', 'DerivativeCheck','off');
	% target function anhysteric
	anonym_func = @(par)fit_static(par, h_static_abs, b_static_abs, weight_s);

% hysteresis
	% options for each local optimization in MultiStart
	ms_options_c = optimset('MaxIter', 70, 'Jacobian','off', 'DerivativeCheck','off');
	% target function
	anonym_func_c = @(par)fit_hysteresis(par, h_up, h_dn, b_up, b_dn, n_par_h);
	
	
	
%% Local fit options
% for static curve
opt_par_s = optimset('MaxIter',250, 'Display','iter', ...
	'TolFun',1e-9, 'Jacobian','on', 'FinDiffType','forward',  ...
	'DerivativeCheck','off');

% for heredity part
opt_par_h = optimset('MaxIter',250, 'Display','iter', ...
	'TolFun',1e-9, 'Jacobian','off', 'FinDiffType','central',  ...
	'DerivativeCheck','off');